{*************
 *
 * Copyright  1997 CEO Systems Inc.
 *
 * Project....: VCL
 * File.......: CEOExcpt.pas
 * Author.....: Alexander V. Losev
 * Date.......: 11 June 1997
 * Description: Class for making and reraising exceptoins
 *
 ************}
 
unit CEOExcpt;

interface

uses
  SysUtils;

  // Create and raise ECEOError exception with passed paramenters
  procedure RaiseCEOError (E: Exception; Info: Array of String);

type
{ ECEOError }
  ECEOError = class(Exception)
  protected
    FComment:   String;
    FClassType: String;
    FCallPath:  String;

  public
    constructor CreateToUp ( E: Exception; const Info: Array of String);
    constructor Create (Msg: String);
    destructor  Destroy; override;

    property    ClassType: String read FClassType;
    property    Comment:   String read FComment;
    property    CallPath:  String read FCallPath;

  end;

implementation

procedure RaiseCEOError (E: Exception; Info: Array of String);
begin
  raise ECEOError.CreateToUp (E, Info);
end;


{*******************************************************************}
{*                       Class ECEOError                           *}
{*******************************************************************}


{*************
* Name.......: CreateToUp
* Description: Create exception to pass futher
* Parameters.: Info can contain [Procname(, Comment)]
*}
constructor ECEOError.CreateToUp (E: Exception; const Info: array of String);
var
  elmNum: Integer;

begin
  elmNum := High(Info);

  if Assigned (E) then          // if no parent exception object
    inherited Create (E.Message)

  else if ( elmNum >= 1 ) then  // init with Comment
    inherited Create (Info[1])

  else // init without something
    inherited Create ('CEO Exception');

  if ( E is ECEOError ) then
    begin
      Self.FCallPath  := ECEOError(E).FCallPath + ' '   + Info[0] + ':';
      // If there is user defined comments in the second position
      if ( elmNum >= 1 ) then
        Self.FComment := ECEOError(E).FComment  + ' | ' + Info[1]
      else
        Self.FComment := ECEOError(E).FComment;
      Self.FClassType := ECEOError(E).FClassType;
    end
  else
    begin
      Self.FCallPath  := Info[0] + ':';
      // If there is user defined comments in the second position
      if ( elmNum >= 1 ) then
        Self.FComment := Info[1]
      else
        Self.FComment := 'CEO Exception';

      // if not NIL then init by the old value
      if Assigned(E) then
        Self.FClassType := E.ClassName
      else
        Self.FClassType := 'CEO Exception';
    end;
end;


{*************
* Name.......: Create
*}
constructor ECEOError.Create (Msg: String);
begin
  inherited Create(Msg);
end;


{*************
* Name.......: Destroy;
*}
destructor ECEOError.Destroy;
begin
  FComment   := '';
  FClassType := '';
  FCallPath  := '';

  inherited Destroy;
end;

end.
