{*************
 *
 * Copyright  1997 CEO Systems Inc.
 *
 * Project....: Crystal Reports VCL
 * File.......: Crystal.pas
 * Author.....: Alexander V. Losev
 * Date.......: 12 June 1997
 * Description: Custom Component for work
 *              with Crystal Reports DLL
 *
 *************}

unit Crystal;

interface

uses
  {CEO}
  crpe32,    CEOExcpt,     StrList,

  {Delphy}
  Windows,   Messages,     SysUtils,    Classes,
  Graphics,  Controls,     Forms,       Dialogs;

const
  // Simple Print Options
  NilOptions: PEPrintOptions =
    (  StructSize:    sizeof (PEPrintOptions);
       StartPageN:    1;
       StopPageN:     0;
       nReportCopies: 1;
       Collation:     PE_DEFAULTCOLLATION
    );

type
  TExecute      = procedure (Sender: TObject; var Cancel: Boolean) of Object;
  TConnect      = procedure (Sender: TObject; var CanInitialize : Boolean) of Object;
  TAfterInit    = procedure (Sender: TObject) of Object;

  ECrystalError = class (ECEOError);

  PEDestination      = (toWindow, toPrinter);//    , toFile); to Future
  TPreviewWindowZoom = (pwFullSize, pwFitOneSize, pwFitBothSides);
  TControlsStyle     = (tcsNew, tcsOld, tcsNone);

  TFileFormats       =  {To Future}
  (
    CrystalReports,     DataInterChange,     WordForWindows,
    WordForDOS,         WordPerfect,         QuattroPro5_0,
    RecordStyle,        RichText,            CommaSeparatedValues,
    TabSeparatedValues, CharSeparatedValues, TextFormat_ASCII,
    PaginatedText,      TabSeparatedText,    Lotus123_WKS,
    Lotus123_WK1,       Lotus123_WK3,        Excel2_1,
    Excel3_0,           Excel4_0,            Excel5_0,
    ODBC,               HTML,                MicrosoftInternetExplorer2_HTML,
    Netscape2_HTML
  );

const

  DLLNames: Array [TFileFormats] of string =  {To Future}
  (
    'u2fcr.dll',   'u2fwordw.dll', 'u2fwordw.dll',
    'u2fdoc.dll',  'u2fdoc.dll',   'u2fqp.dll',
    'u2frec.dll',  'u2frtf.dll',   'u2fsepv.dll',
    'u2fsepv.dll', 'u2fsepv.dll',  'u2ftext.dll',
    'u2ftext.dll', 'u2ftext.dll',  'u2fwks.dll',
    'u2fwks.dll',  'u2fwks.dll',   'u2fxls.dll',
    'u2fxls.dll',  'u2fxls.dll',   'u2fxls.dll',
    'u2fodbc.dll', 'u2fhtml.dll',  '',
    ''
  );

  TypeNames: Array [TFileFormats] of Cardinal =   {To Future}
  (
    UXFCrystalReportType, UXFDIFType,            UXFWordWinType,
    UXFWordDosType,       UXFWordPerfectType,    UXFQP5Type,
    UXFRecordType,        UXFRichTextFormatType, UXFCommaSeparatedType,
    UXFTabSeparatedType,  UXFCharSeparatedType,  UXFTextType,
    UXFPaginatedTextType, UXFTabbedTextType,     UXFLotusWksType,
    UXFLotusWk1Type,      UXFLotusWk3Type,       UXFXls2Type,
    UXFXls3Type,          UXFXls4Type,           UXFXls5Type,
    UXFODBCType,          UXFHTML3Type,          UXFExplorer2Type,
    UXFNetscape2Type
  );

type
  {Class TCRYSTALOptions}
  TCrystalOptions = class(TComponent)

    PrinterSetupDialog: TPrinterSetupDialog;
    //procedure MakeClear(Sender: TObject; var Action: TCloseAction);

  private

    //--------------- Printer Settings
    procedure SetMarginBottom  (Value: Word);
    procedure SetMarginLeft    (Value: Word);
    procedure SetMarginRight   (Value: Word);
    procedure SetMarginTop     (Value: Word);

    procedure SetStartPage     (Value: Word);
    procedure SetStopPage      (Value: Word);
    procedure SetCollation     (Value: Boolean);
    procedure SetPrinterCopies (Value: Word);

    function GetStartPage      : Word;
    function GetStopPage       : Word;
    function GetCollation      : Boolean;
    function GetPrinterCopies  : Word;

    procedure SetPrinterName   (Value: String);
    procedure SetPrinterDriver (Value: String);
    procedure SetPrinterPort   (Value: String);

    //--------------- Window Settings
    procedure SetWindowTitle  (Value: String);
    procedure SetMDIChild     (Value: Boolean);

    function  GetWindowBorderStyle: TFormBorderStyle;
    function  GetWindowState: TWindowState;
    function  GetWindowControlBox : Boolean;
    function  GetWindowMaxButton  : Boolean;
    function  GetWindowMinButton  : Boolean;
    function  GetWindowTop : Word;
    procedure SetWindowTop   (Value: Word);
    function  GetWindowLeft: Word;
    procedure SetWindowLeft  (Value: Word);

    //--------------- Output File Settings
    // To Future

    //--------------- Report Settings
    procedure SetPreview(Value: Boolean);
    function  GetPreview: Boolean;

    procedure SetHeaderH      (Value: Integer);
    procedure SetDetailH      (Value: Integer);
    procedure SetGrandTotalH  (Value: Integer);
    procedure SetFooterH      (Value: Integer);
    procedure SetTitleH       (Value: Integer);
    procedure SetGroupHeaderH (Value: Integer);
    procedure SetGroupFooterH (Value: Integer);

    procedure SetSelectionFormula      (Value: String);
    procedure SetGroupSelectionFormula (Value: String);
    procedure SetReportTitle           (Value: String);
    procedure SetSqlQuery              (Value: String);
    procedure SetDetailCopies          (Value: Integer);
    procedure SetPrintDate             (Value: TDatetime);

    //--------------- Private additional functions
    procedure RealiseProperties(TheHandle: HWnd);
    procedure RealiseRunTimeProperties;

    function  GetError(Const JobIn : Word)          : String;
    procedure Alas(Value: Boolean; ErrCall: String);
    procedure ClearVars;

    function  ExtractFName  (Str: String)     : String;
    function  ExtractFValue (Str: String)     : String;
    function  ExtractSName  (Str: String)     : String;
    function  ExtractSSign  (Str: String)     : Integer;


    procedure   ExecuteOnly; virtual;
    procedure   Execute; virtual;

  private
    MainJobNumber    : Word;
    JobNumber        : Word;
    FIsOpenedJob     : Boolean;
    ErrPath, ErrStr  : String;
    FExecute         : TExecute;

    // Window Settings
    FCustomFormHandle: HWnd;
    FWindowHeight    : Word;              // Init in the constructor (and Clear)
    FWindowWidth     : Word;              // ...
    FWindowTop       : Word;              // ...
    FWindowLeft      : Word;              // ...
    FDWindowLeft     : Word;              // ...
    FDWindowTop      : Word;

    FWindowControls  : TControlsStyle;    // ...

    FWindowControlBox: Boolean;           // ...
    FWindowMaxButton : Boolean;           // ...
    FWindowMinButton : Boolean;           // ...

    FWindowState     : TWindowState;      // Init in the constructor (and Clear)
    FWindowBorderStyle: TFormBorderStyle; // ...
    FWindowTitle     : String;            LWindowTitle       : Boolean;
    FMDIChild        : Boolean;           // Init in the constructor (and Clear)

    FZoomPreviewWindow: TPreviewWindowZoom;

    // Huge Fat Properties
    FFormulas:        TStringListPro;     // Using Count for check editing
    FSortFields:      TStringListPro;
    FDataFiles:       TStringListPro;
    FGroupCondition:  TStringListPro;
    FGroupSortFields: TStringListPro;

    // Report Settings
    FFileName:     String;                // Empty in the constructor (and Clear)

    FSelectionFormula:      String;       LSelectionFormula : Boolean;
    FGroupSelectionFormula: String;       LGroupSelectionFormula : Boolean;
    FReportTitle:           String;       LReportTitle      : Boolean;
    FSQLQuery:              String;       LSQLQuery         : Boolean;
    FDetailCopies:         Integer;       LDetailCopies     : Boolean;
    FDestination:    PEDestination;       // Init  in the constructor (and Clear)
    FPrintDate:          TDateTime;       LPrintDate        : Boolean;
    FEnableProgressDialog: Boolean;       // Init in the constructor  (and Clear)

    FHeaderH        : Integer;            LHeaderH          : Boolean;
    FDetailH        : Integer;            LDetailH          : Boolean;
    FGrandTotalH    : Integer;            LGrandTotalH      : Boolean;
    FFooterH        : Integer;            LFooterH          : Boolean;
    FTitleH         : Integer;            LTitleH           : Boolean;
    FGroupHeaderH   : Integer;            LGroupHeaderH     : Boolean;
    FGroupFooterH   : Integer;            LGroupFooterH     : Boolean;

    // Printer Settings
    FPrintOptions   : PEPrintOptions;     LStartPageN       : Boolean;
                                          LnReportCopies    : Boolean;
                                          LStopPageN        : Boolean;
                                          LCollation        : Boolean;

    FMarginBottom    : Word;              LMarginBottom     : Boolean;
    FMarginLeft      : Word;              LMarginLeft       : Boolean;
    FMarginRight     : Word;              LMarginRight      : Boolean;
    FMarginTop       : Word;              LMarginTop        : Boolean;

    FPrinterName     : String;            LPrinterName      : Boolean;
    FPrinterDriver   : String;            LPrinterDriver    : Boolean;
    FPrinterPort     : String;            LPrinterPort      : Boolean;

    FUserPrinterSetup : Boolean;           // Init in the constructor  (and Clear)
    FUserSetPrtOptions: Boolean;           // Init in the constructor  (and Clear)

    // File Settings
    {To Future}
    FExportOptions: PEExportOptions;      LFormatDLLName             : Boolean;
                                          LFormatType                : Boolean;
                                          LFormatOptions             : Boolean;
                                          LDestinationDLLName        : Boolean;
                                          LDestinationType           : Boolean;
                                          LDestinationOptions        : Boolean;
                                          LnFormatOptionsBytes       : Boolean;
                                          LnDestinationOptionsBytes  : Boolean;

  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy;                     override;
    procedure   Clear;                       
    procedure   SetFocus;
    procedure   NextMagnification;
    function    CalcPagesCount: Smallint;
    function    GetWindowHandle: HWnd;
    procedure   Assign(Value: TCrystalOptions);

    // Especial methods for custom using
    function    GetWindowHandleValue: HWnd;
    function    IsCompleted: Boolean;
    function    ReadStatus: PEJobInfo;
    procedure   BreakOperation;
    procedure   PrintFromWindow;

    procedure   ShowFirst;
    procedure   ShowPrior;
    procedure   ShowNext;
    procedure   ShowLast;
    procedure   ShowPage(Num: Integer);

//
// Report Properties

{+} property HeaderHeight      :Integer  read FHeaderH     write SetHeaderH;
{+} property DetailHeight      :Integer  read FDetailH     write SetDetailH;
{+} property GrandTotalHeight  :Integer  read FGrandTotalH write SetGrandTotalH;
{+} property FooterHeight      :Integer  read FFooterH     write SetFooterH;
{+} property TitleHeight       :Integer  read FTitleH      write SetTitleH;
//{} property GroupHeaderHeight :Integer         read FGroupHeaderH          write SetGroupHeaderH;
//{} property GroupFooterHeight :Integer         read FGroupFooterH          write SetGroupFooterH;

//
// Printer Settings
{+} property PrinterStartPage: Word    read GetStartPage     write SetStartPage;
{+} property PrinterStopPage : Word    read GetStopPage      write SetStopPage;
{+} property PrinterCollation: Boolean read GetCollation     write SetCollation;
{+} property PrinterCopies   : Word    read GetPrinterCopies write SetPrinterCopies;
{+} property CopiesToPrinter : Word    read GetPrinterCopies write SetPrinterCopies;

{+} property MarginBottom    : Word  read FMarginBottom    write SetMarginBottom;
{+} property MarginLeft      : Word  read FMarginLeft      write SetMarginLeft;
{+} property MarginRight     : Word  read FMarginRight     write SetMarginRight;
{+} property MarginTop       : Word  read FMarginTop       write SetMarginTop;

{+} property DetailCopies:     Integer read FDetailCopies          write SetDetailCopies;

//
// Window Information Properties
{+} property WindowBorderStyleValue: TFormBorderStyle
      read GetWindowBorderStyle;
{+} property WindowStateValue: TWindowState
      read GetWindowState;
{+} property WindowControlBoxValue: Boolean read GetWindowControlBox;
{+} property WindowMaxButtonValue:  Boolean read GetWindowMaxButton ;
{+} property WindowMinButtonValue:  Boolean read GetWindowMinButton ;

//
// OutPut File Settings
   {To Future}

//
// Future Properties

// property TableSessionInfo[Index: Integer]

  published

//Main
{+} property IsPreview:    Boolean        read GetPreview    write SetPreview;
{+} property ReportName:   String         read FFileName     write FFileName;

//
// Report Settings
{+} property SelectionFormula:      String read FSelectionFormula      write SetSelectionFormula;
{+} property GroupSelectionFormula: String read FGroupSelectionFormula write SetGroupSelectionFormula;
{+} property ReportTitle:           String read FReportTitle           write SetReportTitle;
{+} property SQLQuery:              String read FSQLQuery              write SetSqlQuery;

{+} property Destination: PEDestination    read FDestination           write FDestination;
{+} property PrintDate:   TDateTime        read FPrintDate             write SetPrintDate;
{+} property EnableProgressDialog: Boolean read FEnableProgressDialog  write FEnableProgressDialog;

//
// Printer Settings
{+} property PrinterName   : String  read FPrinterName     write SetPrinterName;
{+} property PrinterDriver : String  read FPrinterDriver   write SetPrinterDriver;
{+} property PrinterPort   : String  read FPrinterPort     write SetPrinterPort;

{+} property UserPrinterSetup:   Boolean  read FUserPrinterSetup write FUserPrinterSetup;
{+} property UserPrinterOptions: Boolean read FUserSetPrtOptions write FUserSetPrtOptions;

//
// Window Settings
{+} property WindowHeight: Word   read FWindowHeight write FWindowHeight;
{+} property WindowWidth:  Word   read FWindowWidth  write FWindowWidth;

{+} property WindowTop:    Word   read GetWindowTop  write SetWindowTop;
{+} property WindowLeft:   Word   read GetWindowLeft write SetWindowLeft;

{+} property ZoomPreviewWindow: TPreviewWindowZoom read FZoomPreviewWindow write FZoomPreviewWindow;

//
// Window Attributes
{+} property WindowControls:   TControlsStyle read FWindowControls write FWindowControls;

{+} property WindowControlBox: Boolean read FWindowControlBox  write FWindowControlBox;
{+} property WindowMaxButton:  Boolean read FWindowMaxButton   write FWindowMaxButton ;
{+} property WindowMinButton:  Boolean read FWindowMinButton   write FWindowMinButton ;

{+} property WindowTitle:      String  read FWindowTitle write SetWindowTitle;
{+} property MDIChild:         Boolean read FMDIChild    write SetMDIChild;

{+} property WindowBorderStyle: TFormBorderStyle
      read FWindowBorderStyle write FWindowBorderStyle;
{+} property WindowState: TWindowState
      read FWindowState       write FWindowState;

//
// Huge Fat PROPERTIES Previous Version
//{+} property DataFiles       [Index: Integer]: String read GetDFLocation write SetDFLocation;
//{+} property Formulas        [Index: Integer]: String read GetFormulas   write SetFormulas;
//{+} property SortFields      [Index: Integer]: String read GetSortFields write SetSortFields;

// Huge Fat PROPERTIES
{+} property DataFiles:   TStringListPro read FDataFiles  ;
{+} property Formulas :   TStringListPro read FFormulas   ;
{+} property SortFields:  TStringListPro read FSortFields ;

{Not writed only writing to report}
//  property GroupCondition  [Index: Integer]: String read GetGroupCondition  write SetGroupCondition;
//  property GroupSortFields [Index: Integer]: String read GetGroupSortFields write SetGroupSortFields;

//

// Events
{+} property  OnExecute:   TExecute   read FExecute   write FExecute;
//{} property OnConnect:   TConnect   read FConnect   write FExecute;
//{} property OnAfterInit: TAfterInit read FAfterInit write FAfterInit;

//
// Old Design time properties
{
Connect               MDIChild	                SelectionFormula
CopiesToPrinter      	PrinterCollation        	SQLQuery
                      PrinterDriver     	      UserPrinterSetup
Destination  	        PrintFileCharSepSeparator	WindowBorderStyle
DetailCopies	        PrintFileCharSepQuote	    WindowControlBox
DiscardSavedData	    PrintFileName	            WindowControls
EMailCCList	          PrintFileType	            WindowLeft
EMailMessage	        PrintFileUseRptNumberFmt  WindowMaxButton
EMailSubject	        PrintFileUseRptDateFmt	  WindowMinButton
EMailToList	          PrinterCopies	            WindowState
EMailVIMBCCList       PrinterName	              WindowTitle
GroupSelectionFormula	PrinterPort	              WindowHeight
MarginBottom	        PrinterStartPage	        WindowTop
MarginLeft	          PrinterStopPage	          WindowWidth
MarginRight          	ReportName
MarginTop	            ReportTitle}

// Old Run time Properties list
{
Action         	      LastErrorString     	ReportStartPage
DataFiles	            Password            	SectionFont
DataFilesLocation	    PrintControls	        SectionFormat
DialogParentHandle   	PrintDay	            SectionLineHeight
Execute	              PrintEnded	          SectionMinHeight
Formulas	            PrinterMode	          SortFields
GraphOptions	        PrintMonth          	Status
GraphText	            PrintYear	            StoredProcParam
GraphType	            RecordsPrinted	      UserName
GroupCondition	      RecordsRead	          WindowParentHandle
GroupSelectionFormula	RecordsSelected	      ZoomMagnification
GroupSortFields	      ReportDisplayPage    	ZoomPreviewWindow
LastErrorNumber	      ReportLatestPage
}

// Old Methods
{
CloseWindow		        ExportPrintWindow     Focused
PrintWindow           <SetFocus>            <ShowFirstPage>
<ShowLastPage>        <ShowNextPage>        <ShowPreviousPage>
}

end;

{Class TCustomList}
  TCustomList = class (TList)
  private
    function GetItem (Index: Integer): TCrystalOptions;

  public
    destructor Destroy; override;

    procedure Clear;
    procedure Assign (Value: TCustomList; AOwner: TComponent);
    function  Add    (Item:  TCrystalOptions): Integer;
    procedure Insert (Index: Integer; Item: TCrystalOptions);
    procedure Delete (Index: Integer);
    function  Find   (Index: String; var Value: TCrystalOptions): Boolean;

    property Items[Index: Integer]: TCrystalOptions read GetItem; default;
  end;


  {Class TCrystal}
  TCrystal = class (TCrystalOptions)
  private
    FSubRepNames:      TStringListPro;
    FSubreportList:    TCustomList;
    FRepeatFlag:       Boolean;

    function  GetSubReport      (Index: String): TCrystalOptions;

  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy;     override;
    procedure   Execute;    override;
    procedure   ExecuteOnly;override;
    procedure   Clear;
    procedure   MakeSubReportList (var List: TStringList);
    procedure   Assign (Value: TCrystal);

    property    Subreports [Index: String]: TCrystalOptions read GetSubReport;
  end;

procedure Register;
procedure RaiseCrystalError (E: Exception; Info: Array of String);

implementation

uses
  FileCtrl, Custform;

{*************
* Name.......: Register
* Description: Registry the VCL
*}
procedure Register;
begin
  RegisterComponents('CEO', [TCrystal]);
end;


{*************
* Name.......: RaiseCrystalError
* Description: Tavtological to RaiseCEOError
*}
procedure RaiseCrystalError (E: Exception; Info: Array of String);
begin
  raise ECrystalError.CreateToUp (E, Info);
end;


//--------------------------------------------------------------------//
//                            TCrystalOptions                         //
//--------------------------------------------------------------------//

{*************
* Name.......: Create
* Description:
*}
constructor TCrystalOptions.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);

  FFormulas          := TStringListPro.Create;   // Create String Lists
  FSortFields        := TStringListPro.Create;
  FDataFiles         := TStringListPro.Create;
  FGroupCondition    := TStringListPro.Create;
  FGroupSortFields   := TStringListPro.Create;

  // Create Setup dialog
  PrinterSetupDialog := TPrinterSetupDialog.Create(Self);

  FIsOpenedJob  := False;                // Init Global variables
  JobNumber     := 0;
  MainJobNumber := 0;

  ClearVars;  // Call Initialization on default values
end;


{*************
* Name.......: Clear
* Description: Set default Information
*}
procedure TCrystalOptions.Clear;
begin
  if FIsOpenedJob then
  try
    Alas(PECloseSubreport(JobNumber), 'TCrystalOptions.Clear');
  except
    on   E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.Clear');
  end;

  FIsOpenedJob := False;
  ClearVars;
end;


{*************
* Name.......: MakeClear
* Description: Make clearing on close custom preview form
*}
{procedure TCrystalOptions.MakeClear(Sender: TObject; var Action: TCloseAction);
begin
  Clear;
end;}

{*************
* Name.......: ClearVars
* Description: Set default Information
*}
procedure   TCrystalOptions.ClearVars;
begin

  // Reset Flags
  LWindowTitle                  := False;
  LSelectionFormula             := False;
  LGroupSelectionFormula        := False;
  LReportTitle                  := False;
  LSQLQuery                     := False;
  LDetailCopies                 := False;
  LPrintDate                    := False;
  LHeaderH                      := False;
  LDetailH                      := False;
  LGrandTotalH                  := False;
  LFooterH                      := False;
  LTitleH                       := False;
  LGroupHeaderH                 := False;
  LGroupFooterH                 := False;
  LStartPageN                   := False;
  LStopPageN                    := False;
  LCollation                    := False;
  LMarginBottom                 := False;
  LMarginLeft                   := False;
  LMarginRight                  := False;
  LMarginTop                    := False;
  LPrinterName                  := False;
  LPrinterDriver                := False;
  LPrinterPort                  := False;

  {to Future}
{ LFormatDLLName                := False;
  LFormatType                   := False;
  LFormatOptions                := False;
  LDestinationDLLName           := False;
  LDestinationType              := False;
  LDestinationOptions           := False;
  LnFormatOptionsBytes          := False;
  LnDestinationOptionsBytes     := False;
}
  // Set Default Values
  FWindowHeight                 := 400;
  FWindowWidth                  := 600;
  FWindowTop                    := 30;
  FWindowLeft                   := 30;
  FDWindowLeft                  := 0;
  FDWindowTop                   := 0;

  FWindowControlBox             := True;
  FWindowControls               := tcsNew;
  FWindowMaxButton              := True;
  FWindowMinButton              := True;
  FWindowState                  := wsNormal;
  FWindowBorderStyle            := bsSizeable;
  FWindowTitle                  := 'Crystal Reports';
  FMDIChild                     := False;
  FPrintDate                    := Date;
  FUserPrinterSetup             := False;
  FUserSetPrtOptions            := False;
  FCustomFormHandle             := 0;

  // {to Future}
  // Set default Value of Output File Options

  FFileName                     := '';
  FReportTitle                  := '';
  FSQLQuery                     := '';
  FDestination                  := toPrinter;
  FEnableProgressDialog         := True;
  ErrPath                       := '';
  ErrStr                        := '';

  FPrintOptions                 := NilOptions;
  FZoomPreviewWindow            := pwFullSize;

  FFormulas.Clear;
  FSortFields.Clear;
  FDataFiles.Clear;
  FGroupCondition.Clear;
  FGroupSortFields.Clear;
end;


{*************
* Name.......: Destroy
* Description: Free Resource
*}
destructor  TCrystalOptions.Destroy;
begin
  Clear;

  FFormulas.Free;
  FSortFields.Free;
  FDataFiles.Free;
  FGroupCondition.Free;
  FGroupSortFields.Free;
  PrinterSetupDialog.Free;

  inherited Destroy;
end;


{*************
* Name.......: Alas
* Description: Show Error Message if need
*}
procedure TCrystalOptions.Alas(Value: Boolean; ErrCall: String);
begin
  ErrStr  := GetError(JobNumber);
  ErrPath := ErrCall;
  if not Value then
    RaiseCrystalError (nil, [ErrCall, ErrStr]);
end;


{*************
* Name.......: Execute
* Description: Start Print Report
*}
procedure TCrystalOptions.Execute;
begin
  ExecuteOnly;
  if ( (FWindowControls <> tcsNew ) or (FDestination <> toWindow)) then Clear;
end;


{*************
* Name.......: SetFocus
* Description: Set focus on made window
*}
procedure TCrystalOptions.SetFocus;
var
 CurWnd: HWnd;

begin
  if FIsOpenedJob then
  begin
    CurWnd := PEGetWindowHandle (JobNumber);
    Windows.SetFocus(CurWnd);
  end;
end;


{*************
* Name.......: ExecuteOnly
* Description: Start Print Report
*}
procedure TCrystalOptions.ExecuteOnly;
var
  CustomForm: TfrmCrystal;

begin
  {try
    CustomForm := nil;

    if ( FWindowControls = tcsNew ) then
      begin // Showing custom preview form
        if ( FMDIChild and (Owner is TForm) ) then
          CustomForm := TfrmCrystal.Create((Owner as TForm), Self)
        else
          begin
            CustomForm := TfrmCrystal.Create(nil, Self);
            CustomForm.Name := 'frmCrystal' + FormatDateTime('ddhhnnss', Now);
            Application.InsertComponent(CustomForm);
          end;
        FCustomFormHandle := CustomForm.Handle;

        CustomForm.Show;
      end
    else
      begin
        JobNumber  := PEOpenSubReport(MainJobNumber, PChar(FFileName));
        if ( JobNumber = 0 ) then Alas(False, 'TCrystalOptions.Execute');
        FIsOpenedJob := True;

        RealiseProperties(CustomForm);
        Alas(PEStartPrintJob(JobNumber, True), 'TCrystalOptions.Execute');
        while not CustomForm.Made do Application.ProcessMessages;
        RealiseRunTimeProperties;
      end;
  except
    on   E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.Execute');
  end;}
end;


{*************
* Name.......: RealiseProperties
* Description:
*}
procedure   TCrystalOptions.RealiseProperties(TheHandle: HWnd);
var
  I, J, K, L, M: Integer;
  TblPath: PETableLocation;

// Configure Printer
  DrvHdl, PrnHdl, PortHdl: HWnd;
  DrvLen, PrnLen, PortLen: Word;
  CurMode: PDeviceMode;
  CurPrinterDriver, CurPrinterName, CurPrinterPort: String;
  Buffer: PChar;

// Printer options
  CurMarginLeft, CurMarginRight, CurMarginTop, CurMarginBottom: Word;
  CurPrintOptions: PEPrintOptions;

// Configure Window
  PrntWindow:     HWnd;
  CurStyle:       Integer;
  CurWindowTitle: String;

// Configure Output File
   {to Future}
   ExportOptions:   PEExportOptions;

   FDIFOptions                 :  UXFDIFOptions;
   FRecordStyleOptions         :  UXFRecordStyleOptions;
   FCommaTabSeparatedOptions   :  UXFCommaTabSeparatedOptions;
   FCharSeparatedOptions       :  UXFCharSeparatedOptions;
   FPaginatedTextOptions       :  UXFPaginatedTextOptions;
   FODBCOptions                :  UXFODBCOptions;
   FHTML3Options               :  UXFHTML3Options;

 {to Future}
   OptParams:  Array [TFileFormats] of Pointer;

// Report options
   CurWindowsTitle: String;
   Year, Month, Date: Word;

begin
   {To Future}
   {OptParams [CrystalReports                 ] := nil                        ;
    OptParams [DataInterchange                ] := @FDIFOptions               ;
    OptParams [WordForWindows                 ] := nil                        ;
    OptParams [WordForDOS                     ] := nil                        ;
    OptParams [WordPerfect                    ] := nil                        ;
    OptParams [QuattroPro5_0                  ] := nil                        ;
    OptParams [RecordStyle                    ] := @FRecordStyleOptions       ;
    OptParams [RichText                       ] := nil                        ;
    OptParams [CommaSeparatedValues           ] := @FCommaTabSeparatedOptions ;
    OptParams [TabSeparatedValues             ] := @FCommaTabSeparatedOptions ;
    OptParams [CharSeparatedValues            ] := @FCharSeparatedOptions     ;
    OptParams [TextFormat_ASCII               ] := nil                        ;
    OptParams [PaginatedText                  ] := @FPaginatedTextOptions     ;
    OptParams [TabSeparatedText               ] := nil                        ;
    OptParams [Lotus123_WKS                   ] := nil                        ;
    OptParams [Lotus123_WK1                   ] := nil                        ;
    OptParams [Lotus123_WK3                   ] := nil                        ;
    OptParams [Excel2_1                       ] := nil                        ;
    OptParams [Excel3_0                       ] := nil                        ;
    OptParams [Excel4_0                       ] := nil                        ;
    OptParams [Excel5_0                       ] := nil                        ;
    OptParams [ODBC                           ] := @FODBCOptions              ;
    OptParams [HTML                           ] := @FHTML3Options             ;
    OptParams [MicrosoftInternetExplorer2_HTML] := nil                        ;
    OptParams [Netscape2_HTML                 ] := nil                        ;
   }

  {+} {Configure Printer}

  TRY // Catch EXCEPTION

    if ( LPrinterDriver or LPrinterName or LPrinterPort ) then
    begin
      Alas(PEGetSelectedPrinter (JobNumber, DrvHdl, DrvLen, PrnHdl,
        PrnLen, PortHdl, PortLen, CurMode), 'TCrystalOptions.RealiseProperties');

      if LPrinterDriver then
        CurPrinterDriver := FPrinterDriver
      else
        try
          Buffer := StrAlloc(DrvLen);
          Alas(PEGetHandleString(DrvHdl, Buffer, DrvLen), 'TCrystalOptions.RealiseProperties');
          CurPrinterDriver := StrPas(Buffer);
        finally
           StrDispose(Buffer);
        end;

      if LPrinterName   then
        CurPrinterName   := FPrinterName
      else
        try
          Buffer := StrAlloc(PrnLen);
          Alas(PEGetHandleString(PrnHdl, Buffer, PrnLen), 'TCrystalOptions.RealiseProperties');
          CurPrinterName := StrPas(Buffer);
        finally
          StrDispose(Buffer);
        end;

      if LPrinterPort then
        CurPrinterPort   := FPrinterPort
      else
        try
          Buffer := StrAlloc(PortLen);
          Alas(PEGetHandleString(PortHdl, Buffer, PortLen),
            'TCrystalOptions.RealiseProperties');
          CurPrinterPort := StrPas(Buffer);
        finally
          StrDispose(Buffer);
        end;

      Alas(PESelectPrinter(JobNumber, PChar(CurPrinterDriver), PChar(CurPrinterName),
        PChar(CurPrinterPort), CurMode) , 'TCrystalOptions.RealiseProperties');
    end;

    {+}{Set the options for printing}
    if ( LStartPageN or LnReportCopies or LStopPageN or LCollation ) then
    begin
      // Additional initialization
      FPrintOptions.StructSize   := SizeOf (PEPrintOptions);
      CurPrintOptions.StructSize := SizeOf (PEPrintOptions);

      // Get Current Settings
      Alas(PeGetPrintOptions(JobNumber, CurPrintOptions), 'TCrystalOptions.RealiseProperties');

      // ReSet inputed options
      if LStartPageN    then CurPrintOptions.StartPageN    := FPrintOptions.StartPageN;
      if LnReportCopies then CurPrintOptions.nReportCopies := FPrintOptions.nReportCopies;
      if LStopPageN     then CurPrintOptions.StopPageN     := FPrintOptions.StopPageN;
      if LCollation     then CurPrintOptions.Collation     := FPrintOptions.Collation;

      // Write changed options
      Alas(PeSetPrintOptions(JobNumber, CurPrintOptions), 'TCrystalOptions.RealiseProperties');
    end;

    {+}{Set Margins for printing}
    if ( LMarginBottom or LMarginLeft or LMarginRight or LMarginRight ) then
    begin

      // Get Current Settings
      Alas(PeGetMargins(JobNumber, CurMarginLeft, CurMarginRight, CurMarginTop, CurMarginBottom),
      'TCrystalOptions.RealiseProperties');

      // ReSet inputed options
      if LMarginBottom then CurMarginBottom := FMarginBottom;
      if LMarginLeft   then CurMarginLeft   := FMarginLeft;
      if LMarginRight  then CurMarginRight  := FMarginRight;
      if LMarginTop    then CurMarginTop    := FMarginTop;

      // Write changed options
        Alas(PeSetMargins(JobNumber, CurMarginLeft, CurMarginRight, CurMarginTop, CurMarginBottom),
          'TCrystalOptions.RealiseProperties');
    end;

   {Set Report information}

    {+}{Set Minimum Sections Height}
    if LHeaderH then
      Alas(PESetMinimumSectionHeight(JobNumber, PE_HEADERSECTION, FHeaderH),
        'TCrystalOptions.RealiseProperties');
    if LDetailH then
      Alas(PESetMinimumSectionHeight(JobNumber, PE_DETAILSECTION, FDetailH),
        'TCrystalOptions.RealiseProperties');
    if LGrandTotalH then
      Alas(PESetMinimumSectionHeight(JobNumber, PE_GRANDTOTALSECTION, FGrandTotalH),
        'TCrystalOptions.RealiseProperties');
    if LFooterH then
      Alas(PESetMinimumSectionHeight(JobNumber, PE_FOOTERSECTION, FGrandTotalH),
        'TCrystalOptions.RealiseProperties');
    if LTitleH then
      Alas(PESetMinimumSectionHeight(JobNumber, PE_TITLESECTION, FGrandTotalH),
        'TCrystalOptions.RealiseProperties');

  // if LGroupHeaderH then
  //   Alas(PESetMinimumSectionHeight(JobNumber, PE_GROUPHEADER, FGrandTotalH), 'TCrystalOptions.RealiseProperties');
  //  if LGroupFooterH then
  //    Alas(PESetMinimumSectionHeight(JobNumber, PE_GROUPFOOTER, FGrandTotalH), 'TCrystalOptions.RealiseProperties');

    {+}{Count quolity of Database}
    I := PEGetNTables (JobNumber) - 1;
    if ( I = -1 ) then RaiseCrystalError(nil,
      ['TCrystalOptions.RealiseProperties', 'Tables Count Equal 0']);
    if ( FDataFiles.Count < I + 1 ) then // THIS IS PROTECTION
      I := FDataFiles.Count - 1;

    {+}{Set Location of DataBases}
    TblPath.StructSize := PE_SIZEOF_TABLE_LOCATION;
    for J := 0 to I do
    if ( DataFiles[J] <> '' ) then
    begin
      StrPCopy(@(TblPath.Location), DataFiles[J]);
      Alas(PESetNthTableLocation(JobNumber, J, TblPath),
        'TCrystalOptions.RealiseProperties');
    end;

    {+}{Count quolity of Formulas}
    I := PEGetNFormulas(JobNumber) - 1;
    if ( FFormulas.Count < I + 1 ) then // THIS IS PROTECTION
      I := FFormulas.Count - 1;

    {+} {Initialize formulas}
    for J := 0 to I do
      if ( Formulas[J] <> '' ) then
        Alas(PESetFormula(JobNumber, PChar(ExtractFName(Formulas[J])),
          PChar(ExtractFValue(Formulas[J]))), 'TCrystalOptions.RealiseProperties');

    {+}{Count quolity of Sort Fields}
    I := PEGetNSortFields (JobNumber) - 1;
    if ( FSortFields.Count < I + 1 ) then // THIS IS PROTECTION
      I := FSortFields.Count - 1;

    {+}{Initialize Sort Fields}
    for J := 0 to I do  //  '-{stats.STATS DATE}' -> {stats.STATS DATE} & ??
      if ( SortFields[J] <> '' ) then
        Alas(PESetNthSortField (JobNumber, J, PChar(ExtractSName(SortFields[J])),
          ExtractSSign(SortFields[J]) ), 'TCrystalOptions.RealiseProperties');

    {+}{Count quolity of Group }
    I := PEGetNGroups (JobNumber) - 1;
    if ( FGroupCondition.Count < I + 1 ) then // THIS IS PROTECTION
      I := FGroupCondition.Count - 1;

   {to Future}{Initialize Group Conditions}
   { for J := 0 to I do  //
       if ( FGroupCondition <> '' ) then
         if ( PESetGroupCondition (JobNumber, ...) ) then Exit;
   }

    {+}{Count quolity of Sort Fields }
    I := PEGetNGroupSortFields (JobNumber) - 1;
    if ( FGroupSortFields.Count < I + 1 ) then // THIS IS PROTECTION
      I := FGroupSortFields.Count - 1;

    {to Future}{Initialize Group Conditions}
  {  for J := 0 to I do  //
      if ( FGroupSortFields <> '' ) then
        if ( Alas(PESetNthGroupSortField (JobNumber,
  }

    {+} {Set Report Title}
    if LReportTitle then
      Alas(PESetReportTitle(JobNumber, PChar(FReportTitle)),
        'TCrystalOptions.RealiseProperties');

    {+} {Set Selection Formula}
    if LSelectionFormula then
      Alas(PESetSelectionFormula (JobNumber, PChar(FSelectionFormula)),
        'TCrystalOptions.RealiseProperties');

    {+} {Set GroupSelection Formula}
    if LGroupSelectionFormula then
      Alas(PESetGroupSelectionFormula (JobNumber, PChar(FGroupSelectionFormula)),
        'TCrystalOptions.RealiseProperties');

    {+} {Set SQLQuery}
    if LSQLQuery then
      Alas (PESetSQLQuery (JobNumber, PChar(FSQLQuery)),
        'TCrystalOptions.RealiseProperties');

    {+} {Set Detail Copies Count}
    if LDetailCopies then
      Alas(PESetNDetailCopies(JobNumber, FDetailCopies),
        'TCrystalOptions.RealiseProperties');

    {+} {Set Print Date}
    if LPrintDate then
    begin
      DecodeDate(FPrintDate, Year, Month, Date);
      Alas(PESetPrintDate(JobNumber, Year, Month, Date), 'TCrystalOptions.RealiseProperties');
    end;

    {+} {Set Necessity Showing Progress Dialog}
    Alas(PEEnableProgressDialog(JobNumber, FEnableProgressDialog),
      'TCrystalOptions.RealiseProperties');

    {To Future}
    {if ( FDestination = toFile ) then
     begin
       if ( LFormatDLLName       or LFormatType      or LFormatOptions or
            LDestinationDLLName  or LDestinationType or LDestinationOptions or
            LnFormatOptionsBytes or LnDestinationOptionsBytes
          ) then
       begin
         FExportOptions.StructSize := PE_SIZEOF_EXPORT_OPTIONS;
         ...
         PEGetExportOptions
         ..
         PEExportTo
       end;
     end
     else}

    {+}{Set Custom Printer Setup}
    if ( ( (Destination = toPrinter) or
           (FWindowControls = tcsOld)
         ) and FUserPrinterSetup
       ) then PrinterSetupDialog.Execute;

    if ( FDestination = toWindow ) then
      begin
        {+}{Set Window Title}
        if LWindowTitle then  CurWindowsTitle := FWindowTitle;

        {Include/Exclude Print Controls}
        Alas(PEShowPrintControls(JobNumber, (FWindowControls = tcsOld)),
          'TCrystalOptions.RealiseProperties');

        {+}{Set Window Style}
        CurStyle := WS_VISIBLE;// or WS_CAPTION;//
        // Warning! Especially using property but no variable! V13ALX
        if WindowControlBoxValue then CurStyle := (CurStyle or WS_SYSMENU);
        if WindowMaxButtonValue  then CurStyle := (CurStyle or WS_MAXIMIZEBOX);
        if WindowMinButtonValue  then CurStyle := (CurStyle or WS_MINIMIZEBOX);
        // Warning! Especially using property but no variable! V13ALX
        case WindowBorderStyleValue of
          bsNone    : CurStyle := CurStyle;
          bsSingle  : CurStyle := (CurStyle or WS_BORDER);
          bsSizeable: CurStyle := (CurStyle or WS_THICKFRAME);
          bsDialog  : CurStyle := (CurStyle or WS_DLGFRAME);
        end;
        // Warning! Especially using property but no variable! V13ALX
        case WindowStateValue of
          wsNormal   : CurStyle := CurStyle;
          wsMinimized: CurStyle := (CurStyle or WS_MINIMIZE);
          wsMaximized: CurStyle := (CurStyle or WS_MAXIMIZE);
        end;

        {Set Parent Window}
        if ( FWindowControls = tcsNew ) then
          PrntWindow := TheHandle
        else if ( FMDIChild and (Owner is TForm) ) then
          PrntWindow := (Owner as TForm).Handle
        else
          PrntWindow := 0;

        {Set inputed Window Options}
        // Warning! Especially using property but no variable! V13ALX
        Alas(PEOutputToWindow(JobNumber, PChar(FWindowTitle),
          WindowLeft, WindowTop, WindowWidth, WindowHeight, CurStyle, PrntWindow),
            'TCrystalOptions.RealiseProperties');
      end

    else if ( FDestination = toPrinter ) then
      begin
        {To Future: Make Ask selection of pages to print}

        if LnReportCopies then
          Alas(PEOutputToPrinter(JobNumber, CopiesToPrinter), 'TCrystalOptions.RealiseProperties')
        else
          Alas(PEOutputToPrinter(JobNumber, 1), 'TCrystalOptions.RealiseProperties');
      end;

  EXCEPT
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.RealiseProperties');
  END;
end;


{*************
* Name.......: RealiseRunTimeProperties;
*}
procedure TCrystalOptions.RealiseRunTimeProperties;
var
  I: Integer;

begin
  try
    if ( FDestination = toWindow ) then
    for  I := 0 to Ord(FZoomPreviewWindow) - 1 do
      Alas(PENextPrintWindowMagnification(JobNumber),
        'TCrystalOptions.RealiseRunTimeProperties');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.RealiseRunTimeProperties');
  end;

end;


{*************
* Name.......: NextMagnification
* Description: do Next Magnification
*}
procedure TCrystalOptions.NextMagnification;
begin
  try
    if ( FDestination = toWindow ) then
      Alas(PENextPrintWindowMagnification(JobNumber),
        'TCrystalOptions.RealiseNextMagnification');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.RealiseNextMagnification');
  end;
end;


{*************
* Name.......: ShowFirst
* Description: Show First page
*}
procedure   TCrystalOptions.ShowFirst;
begin
  try
    if ( FDestination = toWindow ) then
      Alas( PEShowFirstPage(JobNumber), 'TCrystalOptions.ShowFirst');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.ShowFirst');
  end;
end;


{*************
* Name.......: ShowPrior
* Description: Show Prior page
*}
procedure TCrystalOptions.ShowPrior;
begin
  try
    if ( FDestination = toWindow ) then
      Alas(PEShowPreviousPage(JobNumber), 'TCrystalOptions.ShowPrior');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.ShowPrior');
  end;
end;


{*************
* Name.......: ShowNext
* Description: Show Next page
*}
procedure TCrystalOptions.ShowNext;
begin
  try
    if ( FDestination = toWindow ) then
      Alas(PEShowNextPage(JobNumber), 'TCrystalOptions.ShowNext');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.ShowNext');
  end;
end;


{*************
* Name.......: ShowLast
* Description: Show Last page
*}
procedure TCrystalOptions.ShowLast;
begin
  try
    if ( FDestination = toWindow ) then
      Alas(PEShowLastPage(JobNumber), 'TCrystalOptions.ShowLast');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.ShowLast');
  end;
end;


{*************
* Name.......: ShowPage
* Description: Show selected page
*}
procedure TCrystalOptions.ShowPage(Num: Integer);
begin
  try
    if ( FDestination = toWindow ) then
      Alas(PEShowNthPage(JobNumber, Num), 'TCrystalOptions.ShowPage');
  except
    on E: ECrystalError do raise
    else Alas(False, 'TCrystalOptions.ShowPage');
  end;
end;


{*************
* Name.......: IsCompleted
* Description: check end of process
*}
function TCrystalOptions.IsCompleted: Boolean;
begin
  Result := False;
  try
    Result := PEIsPrintJobFinished(JobNumber);
  except
    on E: Exception do
      RaiseCrystalError (nil, ['TCrystalOptions.IsCompleted', E.Message]);
  end;
end;

{*************
* Name.......: ReadStatus
* Description: Read process information
*}
function TCrystalOptions.ReadStatus: PEJobInfo;
var
  I: Integer;

begin
  Result.StructSize := SizeOf (PEJobInfo);
  try
    I := PEGetJobStatus (JobNumber, Result);
    Result.StructSize := I;  // return addition information
  except
    on E: Exception do
      RaiseCrystalError (nil, ['TCrystalOptions.ReadStatus', E.Message]);
  end;
end;


{*************
* Name.......: BreakOperation
* Description: Attempt break current operation
*}
procedure  TCrystalOptions.BreakOperation;
begin
  try
    PECancelPrintJob(JobNumber);
  except
    Alas(False, 'TCrystalOptions.BreakOperation');
  end;
end;


{*************
* Name.......: PrintFromWindow
* Description: Print from custom preview Window
*}
procedure  TCrystalOptions.PrintFromWindow;
var
  CurPrintOptions: PEPrintOptions;

begin
  {+}{Set the options for printing}
  if ( LStartPageN or LnReportCopies or LStopPageN or LCollation ) then
  begin
    // Additional initialization
    FPrintOptions.StructSize   := SizeOf (PEPrintOptions);
    CurPrintOptions.StructSize := SizeOf (PEPrintOptions);

    // Get Current Settings
    Alas(PeGetPrintOptions(JobNumber, CurPrintOptions), 'TCrystalOptions.PrintFromWindow');

    // ReSet inputed options
    if LStartPageN    then CurPrintOptions.StartPageN    := FPrintOptions.StartPageN;
    if LnReportCopies then CurPrintOptions.nReportCopies := FPrintOptions.nReportCopies;
    if LStopPageN     then CurPrintOptions.StopPageN     := FPrintOptions.StopPageN;
    if LCollation     then CurPrintOptions.Collation     := FPrintOptions.Collation;

    // Write changed options
    Alas(PeSetPrintOptions(JobNumber, CurPrintOptions), 'TCrystalOptions.PrintFromWindow');
  end;

  // printing
  Alas(PEPrintWindow (JobNumber, False), 'TCrystalOptions.PrintFromWindow');
end;

{*************
* Name.......: CalcPagesCount
* Description: Attempt Calculate count of pages by library (not correct in some cases)
*}
function TCrystalOptions.CalcPagesCount: Smallint;
begin
  try
    Result := PEGetNPages(JobNumber);
  except
    Alas(False, 'TCrystalOptions.CalcPagesCount');
  end;
end;


{*************
* Name.......: GetWindowHandleValue
* Description: Read current (not real) window handle
*}
function TCrystalOptions.GetWindowHandleValue: HWnd;
begin
  try
    if ( FIsOpenedJob and (Destination = toWindow) ) then
      begin
        if ( FWindowControls = tcsNew ) then
          Result := FCustomFormHandle
        else
          Result := PEGetWindowHandle (JobNumber)
      end
    else
      Result := 0;
  except
    Result := 0;
    Alas(False, 'TCrystalOptions.GetWindowHandleValue');
  end;
end;


{*************
* Name.......: GetWindowHandle
* Description: Read real window handle
*}
function TCrystalOptions.GetWindowHandle: HWnd;
begin
  try
    if ( FIsOpenedJob and (Destination = toWindow) ) then
      Result := PEGetWindowHandle (JobNumber)
    else
      Result := 0;
  except
    Result := 0;
    Alas(False, 'TCrystalOptions.GetWindowHandleValue');
  end;
end;

//   --------------------------
//   Additional Private Methods
//   --------------------------

{*************
* Name.......: GetError
* Description: Show Error Message
*}
function TCrystalOptions.GetError(Const JobIn : Word) : String;
{This is my print engine error message capture function. It accepts
 the job number as it parameter and then gets the error code and
 message text and then passes this back out as a formatted string}
var
  Code:         Integer;
  Length:       Word;
  StrHandle:    hWnd;
  Buffer:       PChar;
  Ret :         Bool;

begin

   Code   := PEGetErrorCode(JobIn);                       {Get the Error code from the Crpe}
   Ret    := PEGetErrorText(JobIn, StrHandle, Length);    {Get the error message handle}
   Buffer := StrAlloc(Length);
   try     {get the text from the text handle}
     Ret      := PEGetHandleString(StrHandle, Buffer, Length);
     GetError := IntToStr(Code) + ' - ' + StrPas(Buffer); {output the string}
   finally
     StrDispose(Buffer);
   end;
end;


{*************
* Name.......: ExtractFName
* Description: Define Name of Formula
*}
function TCrystalOptions.ExtractFName  (Str: String): String;
var
  I: Integer;

begin
  I := Pos('=', Str);
  if ( I = 0 ) then
    Result := ''
  else
    Result := Copy(Str, 1, I - 1);
end;


{*************
* Name.......: ExtractFValue
* Description: Define value of Formula
*}
function TCrystalOptions.ExtractFValue (Str: String): String;
var
  I: Integer;

begin
  I := Pos('=', Str);
  if ( I = 0 ) then
    Result := ''
  else
    Result := Copy(Str, I + 1, Length(Str) - I);
end;


{*************
* Name.......: ExtractSName
* Description: Define Sort Field
*}
function TCrystalOptions.ExtractSName  (Str: String): String;
begin
  if ( Str[1] = '-' ) then
    Result := Copy(Str, 2, Length(Str) - 1)
  else if ( Str[1] = '{' ) then
    Result := Str;
end;


{*************
* Name.......: ExtractSSign
* Description: Define order of sorting
*}
function TCrystalOptions.ExtractSSign  (Str: String): Integer;
begin
  if Str[1] = '-' then
    Result := PE_SF_DESCENDING
  else if ( Str[1] = '{' ) then
    Result := PE_SF_ASCENDING
  else
    Result := -1;
end;

// -----------Printer Settings-------------------
//------------Set Margins

procedure TCrystalOptions.SetMarginBottom (Value: Word);
begin
  FMarginBottom := Value;
  LMarginBottom := True;
end;

procedure TCrystalOptions.SetMarginLeft  (Value: Word);
begin
  FMarginLeft := Value;
  LMarginLeft := True;
end;

procedure TCrystalOptions.SetMarginRight (Value: Word);
begin
  FMarginRight := Value;
  LMarginRight := True;
end;

procedure TCrystalOptions.SetMarginTop (Value: Word);
begin
  FMarginTop := Value;
  LMarginTop := True;
end;

//------------Set Print Options

procedure TCrystalOptions.SetStartPage (Value: Word);
begin
 FPrintOptions.StartPageN := Value;
 LStartPageN              := True;
end;

procedure TCrystalOptions.SetStopPage (Value: Word);
begin
 FPrintOptions.StopPageN := Value;
 LStopPageN              := True;
end;

procedure TCrystalOptions.SetCollation (Value: Boolean);
begin
 if Value then
   FPrintOptions.Collation   := PE_COLLATED
 else
   FPrintOptions.Collation   := PE_UNCOLLATED;

 LCollation                := True;
end;

procedure TCrystalOptions.SetPrinterCopies (Value: Word);
begin
 FPrintOptions.nReportCopies := Value;
 LnReportCopies              := True;
end;

//------------Get Print Options

function TCrystalOptions.GetStartPage     : Word;
begin
  Result := FPrintOptions.StartPageN;
end;

function TCrystalOptions.GetStopPage      : Word;
begin
  Result := FPrintOptions.StopPageN;
end;

function TCrystalOptions.GetCollation     : Boolean;
begin
  case FPrintOptions.Collation of
    PE_UNCOLLATED: Result := False;
    PE_COLLATED: 	 Result := True;
    else Result := False;
  end;
end;

function TCrystalOptions.GetPrinterCopies : Word;
begin
  Result := FPrintOptions.nReportCopies;
end;

//--------------Configure Printer

procedure TCrystalOptions.SetPrinterName (Value: String);
begin
  FPrinterName := Value;
  LPrinterName := True;
end;

procedure TCrystalOptions.SetPrinterDriver (Value: String);
begin
  FPrinterDriver := Value;
  LPrinterDriver := True;
end;

procedure TCrystalOptions.SetPrinterPort (Value: String);
begin
  FPrinterPort := Value;
  LPrinterPort := True;
end;

//------------------
//WindowSettings

procedure TCrystalOptions.SetWindowTitle (Value: String);
begin
  FWindowTitle := Value;
  LWindowTitle := True;
end;

procedure TCrystalOptions.SetMDIChild (Value: Boolean);
begin
  if ( Owner is TForm ) then
    FMDIChild := Value
  else
    FMDIChild := False;
end;

function TCrystalOptions.GetWindowBorderStyle: TFormBorderStyle;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowBorderStyle
  else
    Result := bsNone;
end;

function TCrystalOptions.GetWindowState: TWindowState;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowState
  else
    Result := wsNormal;
end;

function  TCrystalOptions.GetWindowTop: Word;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowTop
  else
    Result := FDWindowTop;
end;

procedure TCrystalOptions.SetWindowTop (Value: Word);
begin
  if ( FWindowControls <> tcsNew ) then
    FWindowTop := Value
  else
    FDWindowTop := Value;
end;

function  TCrystalOptions.GetWindowLeft: Word;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowLeft
  else
    Result := FDWindowLeft;
end;

procedure TCrystalOptions.SetWindowLeft (Value: Word);
begin
  if ( FWindowControls <> tcsNew ) then
    FWindowLeft := Value
  else
    FDWindowLeft := Value;
end;

function  TCrystalOptions.GetWindowControlBox : Boolean;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowControlBox
  else
    Result := False;
end;

function  TCrystalOptions.GetWindowMaxButton  : Boolean;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowMaxButton
  else
    Result := False;
end;

function  TCrystalOptions.GetWindowMinButton  : Boolean;
begin
  if ( FWindowControls <> tcsNew ) then
    Result := FWindowMinButton
  else
    Result := False;
end;

//--------------
// OutPut File Settings
   {To Future}

//--------------
// Report Settings

procedure TCrystalOptions.SetPreview (Value: Boolean);
begin
  if Value then FDestination := toWindow
           else FDestination := toPrinter;
end;

function TCrystalOptions.GetPreview: Boolean;
begin
  Result := (FDestination = toWindow);
end;

procedure TCrystalOptions.SetHeaderH (Value: Integer);
begin
  FHeaderH := Value;
  LHeaderH := True;
end;

procedure TCrystalOptions.SetDetailH (Value: Integer);
begin
  FDetailH := Value;
  LDetailH := True;
end;

procedure TCrystalOptions.SetGrandTotalH (Value: Integer);
begin
  FGrandTotalH := Value;
  LGrandTotalH := True;
end;

procedure TCrystalOptions.SetFooterH (Value: Integer);
begin
  FFooterH := Value;
  LFooterH := True;
end;

procedure TCrystalOptions.SetTitleH (Value: Integer);
begin
  FTitleH := Value;
  LTitleH := True;
end;

procedure TCrystalOptions.SetGroupHeaderH (Value: Integer);
begin
  FGroupHeaderH := Value;
  LGroupHeaderH := True;
end;

procedure TCrystalOptions.SetGroupFooterH (Value: Integer);
begin
  FGroupFooterH := Value;
  LGroupFooterH := True;
end;

procedure TCrystalOptions.SetReportTitle (Value: String);
begin
  FReportTitle := Value;
  LReportTitle := True;
end;

procedure TCrystalOptions.SetSelectionFormula (Value: String);
begin
  FSelectionFormula := Value;
  LSelectionFormula := True;
end;

procedure TCrystalOptions.SetGroupSelectionFormula (Value: String);
begin
  FGroupSelectionFormula := Value;
  LGroupSelectionFormula := True;
end;


procedure TCrystalOptions.SetSQLQuery (Value: String);
begin
  FSQLQuery := Value;
  LSQLQuery := True;
end;

procedure TCrystalOptions.SetDetailCopies (Value: Integer);
begin
  FDetailCopies := Value;
  LDetailCopies := True;
end;

procedure TCrystalOptions.SetPrintDate (Value: TDatetime);
begin
  FPrintDate := Value;
  LPrintDate := True;
end;


//----------------------------------------------------------------------//
//                        TCrystal                                      //
//----------------------------------------------------------------------//

{*************
* Name.......: Create
*}
constructor TCrystal.Create (AOwner: TComponent);
begin
  TRY
    inherited Create (AOwner);

    FSubRepNames    := TStringListPro.Create;
    FSubreportList  := TCustomList.Create;

    {+}{Open the Crystal Print Engine}
    if not PEOpenEngine then
      RaiseCrystalError(nil, ['TCrystal.Create', 'Print Engine Not Opened']);
  EXCEPT
    on E: ECrystalError do raise
    else Alas(False, 'TCrystal.Create');
  END;

end;


{*************
* Name.......: Destroy
* Description: Free Resources
*}
destructor TCrystal.Destroy;
begin
  Clear;

  FSubRepNames.Free;
  FSubreportList.Free;

  PECloseEngine;
  inherited Destroy;
end;


{*************
* Name.......: GetSubReport
*}
function TCrystal.GetSubReport (Index: String): TCrystalOptions;
var
  TMP: TCrystalOptions;

begin
  // Check if subreport was selected on previous call
  if not FSubreportList.Find (Index, Result) then
  begin
    TMP := TCrystalOptions.Create(Owner);
    TMP.ReportName := Index;
    FSubreportList.Add (TMP);
    Result := TMP;
  end;
end;


{*************
* Name.......: Execute
* Description: Print/Show report
*}
procedure TCrystal.Execute;
begin
  ExecuteOnly;
  if ( (FWindowControls <> tcsNew ) or (FDestination <> toWindow)) then Clear;
  {else if ( FCustomFormHandle <> 0 ) then
    (FindControl(FCustomFormHandle) as TfrmCrystal).OnClose := MakeClear;}
end;


{*************
* Name.......: ExecuteOnly
* Description: Print/Show report
*}
procedure TCrystal.ExecuteOnly;
var
  I: Integer;
  List : TStringList;
  B: String;
  CanCancel: Boolean;
  CustomForm: TfrmCrystal;
  XRes: Boolean;

begin
  if Assigned (FExecute) then
  begin
    FExecute(Self, CanCancel);
    if CanCancel then Exit;
  end;

  try
    // Creation custom form
    if ( (FDestination = toWindow) and (FWindowControls = tcsNew) and not FRepeatFlag ) then
      begin
        CustomForm := nil;
        FRepeatFlag := True;
        if ( FMDIChild and (Owner is TForm) ) then
          CustomForm := TfrmCrystal.Create((Owner as TForm), Self)
        else
          begin
            CustomForm := TfrmCrystal.Create(Application{nil}, Self);
//          CustomForm.Name := 'frmCrystal' + FormatDateTime('ddhhnnss', Now);
//          Application.InsertComponent(CustomForm);
          end;
        FCustomFormHandle := CustomForm.Handle;
        CustomForm.Crystal1.FCustomFormHandle := FCustomFormHandle;
        CustomForm.Show;
        while not CustomForm.Made do Application.ProcessMessages;
      end
    else
      begin
        {+}{Open File}
        JobNumber := PEOpenPrintJob(PChar(FFileName));
        Alas( JobNumber <> 0, 'TCrystal.Execute');
        FIsOpenedJob := True; // Set file-is-open Flag

        for I := 0 to FSubreportList.Count - 1 do
        begin
          FSubreportList[I].JobNumber := PEOpenSubReport(JobNumber,
            PChar(FSubreportList[I].ReportName));
          FSubreportList[I].MainJobNumber := JobNumber;
          if ( FSubreportList.Items[I].JobNumber <> 0 ) then
            FSubreportList[I].RealiseProperties(FCustomFormHandle)
          else
            RaiseCrystalError(nil, ['TCrystal.Execute', 'Can not Open Subreport '
              + FSubreportList[I].ReportName]);
        end;

        RealiseProperties(FCustomFormHandle);
        XRes := PEStartPrintJob(JobNumber, True);
        if not XRes then
          RaiseCrystalError (nil, ['TCrystalOptions.ExecuteOnly', 'Print Job not Started']);
        RealiseRunTimeProperties;
      end;
  EXCEPT
    on   E: ECrystalError do raise;
    else Alas(False, 'TCrystal.ExecuteOnly');
  END;
end;


{*************
* Name.......: Clear
* Description: Clear Class
*}
procedure TCrystal.Clear;
var
  I,J : Integer;

begin
  TRY
    FSubRepNames.Clear;

    if FIsOpenedJob then
    begin
      J := FSubreportList.Count - 1;
      for I := 0 to J do
        FSubreportList[J].Clear;

      Alas(PEClosePrintJob(JobNumber), 'TCrystal.Clear');
    end;
    FIsOpenedJob := False;
    FSubreportList.Clear;

    inherited ClearVars;
    FRepeatFlag := False;

  EXCEPT
    on   E: ECrystalError do raise
    else Alas(False, 'TCrystal.Clear');
  END;
end;


{*************
* Name.......: MakeSubReportList
* Description: Search all SubReport Names
*}
procedure TCrystal.MakeSubReportList (var List: TStringList);
var
  SecCode, NSubrep, NumSect: Integer;
  NthRepCode:                Cardinal;
  SubInfo:                   PESubreportInfo;
  SubRepName:                String;
  SubRepJob:                 Word;
  I,J:                       Integer;

begin
  TRY
    List.Clear;
    SubInfo.StructSize := PE_SIZEOF_SUBREPORT_INFO;            //Set Init value

    NumSect := PEGetNSections(JobNumber);                      // Get Count of Sections
    if ( NumSect = -1 ) then                                   // Out of Sections
      RaiseCrystalError (nil, ['TCrystal.MakeSubReportList', 'Error Reading Sections Count']);

    for J := 0 to (NumSect - 1) do                             // By Sections
    begin
      SecCode := PEGetSectionCode(JobNumber, J);               // Get Section ID
      NSubrep := PEGetNSubreportsInSection(JobNumber, SecCode);// Count Subreports in section
      if ( NSubrep = -1 ) then                                   // Break on Error
        RaiseCrystalError (nil, ['TCrystal.MakeSubReportList',
          'Error Reading Subreport Count']);

      for I := 0 to (NSubRep - 1) do                           // By Subreports
      begin
        NthRepCode := PEGetNthSubreportInSection(JobNumber, SecCode, I); // Get report ID
        Alas(PEGetSubreportInfo(JobNumber, NthRepCode, SubInfo),  // Get SubReport Info
          'TCrystal.MakeSubReportList');
        SubRepName := Strpas(@(SubInfo.Name));                   // Decode SubReport Info
        List.Add(SubRepName);                          // Update List
      end;
    end;

  EXCEPT
    on   E: ECrystalError do raise
    else Alas(False, 'TCrystal.MakeSubReportList');
  END;
end;
                
procedure TCrystalOptions.Assign(Value: TCrystalOptions);
begin
  Self.MainJobNumber           :=      Value.MainJobNumber           ;
  Self.JobNumber               :=      Value.JobNumber               ;
  Self.FIsOpenedJob            :=      Value.FIsOpenedJob            ;
  Self.ErrPath                 :=      Value.ErrPath                 ;
  Self.ErrStr                  :=      Value.ErrStr                  ;
  Self.FExecute                :=      Value.FExecute                ;
  Self.FCustomFormHandle       :=      Value.FCustomFormHandle       ;
  Self.FWindowHeight           :=      Value.FWindowHeight           ;
  Self.FWindowWidth            :=      Value.FWindowWidth            ;
  Self.FWindowTop              :=      Value.FWindowTop              ;
  Self.FWindowLeft             :=      Value.FWindowLeft             ;
  Self.FDWindowLeft            :=      Value.FDWindowLeft            ;
  Self.FDWindowTop             :=      Value.FDWindowTop             ;
  Self.FWindowControls         :=      Value.FWindowControls         ;
  Self.FWindowControlBox       :=      Value.FWindowControlBox       ;
  Self.FWindowMaxButton        :=      Value.FWindowMaxButton        ;
  Self.FWindowMinButton        :=      Value.FWindowMinButton        ;
  Self.FWindowState            :=      Value.FWindowState            ;
  Self.FWindowBorderStyle      :=      Value.FWindowBorderStyle      ;
  Self.FWindowTitle            :=      Value.FWindowTitle            ;
  Self.LWindowTitle            :=      Value.LWindowTitle            ;
  Self.FMDIChild               :=      Value.FMDIChild               ;
  Self.FZoomPreviewWindow      :=      Value.FZoomPreviewWindow      ;
  Self.FFileName               :=      Value.FFileName               ;
  Self.FSelectionFormula       :=      Value.FSelectionFormula       ;
  Self.LSelectionFormula       :=      Value.LSelectionFormula       ;
  Self.FGroupSelectionFormula  :=      Value.FGroupSelectionFormula  ;
  Self.LGroupSelectionFormula  :=      Value.LGroupSelectionFormula  ;
  Self.FReportTitle            :=      Value.FReportTitle            ;
  Self.LReportTitle            :=      Value.LReportTitle            ;
  Self.FSQLQuery               :=      Value.FSQLQuery               ;
  Self.LSQLQuery               :=      Value.LSQLQuery               ;
  Self.FDetailCopies           :=      Value.FDetailCopies           ;
  Self.LDetailCopies           :=      Value.LDetailCopies           ;
  Self.FDestination            :=      Value.FDestination            ;
  Self.FPrintDate              :=      Value.FPrintDate              ;
  Self.LPrintDate              :=      Value.LPrintDate              ;
  Self.FEnableProgressDialog   :=      Value.FEnableProgressDialog   ;
  Self.FHeaderH                :=      Value.FHeaderH                ;
  Self.FDetailH                :=      Value.FDetailH                ;
  Self.FGrandTotalH            :=      Value.FGrandTotalH            ;
  Self.FFooterH                :=      Value.FFooterH                ;
  Self.FTitleH                 :=      Value.FTitleH                 ;
  Self.FGroupHeaderH           :=      Value.FGroupHeaderH           ;
  Self.FGroupFooterH           :=      Value.FGroupFooterH           ;
  Self.LHeaderH                :=      Value.LHeaderH                ;
  Self.LDetailH                :=      Value.LDetailH                ;
  Self.LGrandTotalH            :=      Value.LGrandTotalH            ;
  Self.LFooterH                :=      Value.LFooterH                ;
  Self.LTitleH                 :=      Value.LTitleH                 ;
  Self.LGroupHeaderH           :=      Value.LGroupHeaderH           ;
  Self.LGroupFooterH           :=      Value.LGroupFooterH           ;
  Self.FPrintOptions           :=      Value.FPrintOptions           ;
  Self.LStartPageN             :=      Value.LStartPageN             ;
  Self.LnReportCopies          :=      Value.LnReportCopies          ;
  Self.LStopPageN              :=      Value.LStopPageN              ;
  Self.LCollation              :=      Value.LCollation              ;
  Self.FMarginBottom           :=      Value.FMarginBottom           ;
  Self.FMarginLeft             :=      Value.FMarginLeft             ;
  Self.FMarginRight            :=      Value.FMarginRight            ;
  Self.FMarginTop              :=      Value.FMarginTop              ;
  Self.LMarginBottom           :=      Value.LMarginBottom           ;
  Self.LMarginLeft             :=      Value.LMarginLeft             ;
  Self.LMarginRight            :=      Value.LMarginRight            ;
  Self.LMarginTop              :=      Value.LMarginTop              ;
  Self.FPrinterName            :=      Value.FPrinterName            ;
  Self.FPrinterDriver          :=      Value.FPrinterDriver          ;
  Self.FPrinterPort            :=      Value.FPrinterPort            ;
  Self.LPrinterName            :=      Value.LPrinterName            ;
  Self.LPrinterDriver          :=      Value.LPrinterDriver          ;
  Self.LPrinterPort            :=      Value.LPrinterPort            ;
  Self.FUserPrinterSetup       :=      Value.FUserPrinterSetup       ;
  Self.FUserSetPrtOptions      :=      Value.FUserSetPrtOptions      ;
                                                                     
  Self.FFormulas.Assign(Value.FFormulas);
  Self.FSortFields.Assign(Value.FSortFields);
  Self.FDataFiles.Assign(Value.FDataFiles);
  Self.FGroupCondition.Assign(Value.FGroupCondition);
  Self.FGroupSortFields.Assign(Value.FGroupSortFields);
end;


procedure TCrystal.Assign(Value: TCrystal);
begin
  inherited Assign(Value);
  Self.FSubRepNames.Assign(Value.FSubRepNames);
  Self.FSubreportList.Assign(Value.FSubreportList, Self.Owner);
  Self.FRepeatFlag := Value.FRepeatFlag;
end;


//----------------------------------------------------------------------//
//                         Class TCustomList                            //
//----------------------------------------------------------------------//


{*************
* Name.......: Destroy
* Description: Free Resources
*}
destructor TCustomList.Destroy;
begin
  Clear;
  inherited Destroy;
end;


{*************
* Name.......: GetItem
* Description: Get Object From List
*}
function TCustomList.GetItem (Index: Integer): TCrystalOptions;
begin
  Result := TCrystalOptions(inherited Items[Index]);
end;


{*************
* Name.......: Clear
* Description: Free objects from list
*}
procedure  TCustomList.Clear;
var
  I: Integer;

begin
  // Free all items stored in the list
  if ( Count <= 0 ) then Exit;
  // delete Items
  while ( Count > 0 ) do Delete (0);
  // perform inherited clearing
  inherited Clear;
end;


{*************
* Name.......: Add
* Description: Add new object
*}
function   TCustomList.Add (Item: TCrystalOptions): Integer;
begin
  Result := inherited Add (Item);
end;


{*************
* Name.......: Find
* Description: Find object by Index
*}
function TCustomList.Find (Index: String; var Value: TCrystalOptions): Boolean;
var
  I: Integer;

begin
  Result := False;
  for I := 0 to Count - 1 do
  if ( (inherited Items[I] <> nil) and (TCrystalOptions(inherited Items[I]).ReportName = Index) ) then
  begin
    Result := True;
    Value  := TCrystalOptions(inherited Items[I]);
    Break;
  end
end;


{*************
* Name.......: Insert
* Description: Insert object to list
*}
procedure  TCustomList.Insert (Index: Integer; Item: TCrystalOptions);
begin
  inherited Insert (Index, Item);
end;


{*************
* Name.......: Delete
* Description: Delete object from list
*}
procedure  TCustomList.Delete (Index: Integer);
begin
  Items[Index].Free;
  inherited Delete (Index);
end;

{*************
* Name.......:  Assign
*}
procedure TCustomList.Assign (Value: TCustomList; AOwner: TComponent);
var
  I,J : Integer;

begin
  Clear;
  J := Value.Count - 1;
  for I := 0 to J do
  begin
    Add(TCrystalOptions.Create(AOwner));
    Items[I].Assign(Value.Items[I]);
  end;
end;

end.

