{*************
 *
 * Copyright  1997 CEO Systems Inc.
 *
 * Project....: Crystal
 * File.......: CustForm.pas
 * Author.....: Alexander V. Losev
 * Date.......: 26/08/1997
 * Description: Custom form for work with Crystal Engine
 * 
 *************}

unit CustForm;

interface

uses
  Windows,  Messages, SysUtils, Classes,
  Graphics, Controls, Forms,    Dialogs,
  StdCtrls, Buttons,  ExtCtrls,
  Crystal, CeoSpin,   TaskBar,
  RXCtrls,  Spin;

type
  TfrmCrystal = class(TForm)
    pnlControls: TPanel;

    btnNext:    TRxSpeedButton;
    btnBreak:   TRxSpeedButton;
    btnFirst:   TRxSpeedButton;
    btnPrior:   TRxSpeedButton;
    btnPrint:   TRxSpeedButton;
    btnMagn:    TRxSpeedButton;
    btnExport:  TRxSpeedButton;
    btnGo:      TRxSpeedButton;
    btnSpLast:  TRxSpeedButton;
    btnSpBreak: TSpeedButton;

    lblCurRec: TLabel;
    lblPages:  TLabel;
    lblPage:   TLabel;
    lblProc:   TLabel;
    lblInfo:   TLabel;
    lblTotal:  TLabel;

    Timer1:              TTimer;
    seNum:               TCEOSpinEdit;
    PrintDialog1:        TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;

    procedure btnPrintClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnMagnClick(Sender: TObject);

    procedure btnFirstClick(Sender: TObject);
    procedure btnPriorClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnSpLastClick(Sender: TObject);
    procedure btnSetPageClick(Sender: TObject);
    procedure btnBreakClick(Sender: TObject);

    procedure TimeMoving(Sender: TObject);
    procedure TimeReading(Sender: TObject);
    procedure TimeBreaking(Sender: TObject);
    procedure TimeActivating(Sender: TObject);
    procedure TimePrinting(Sender: TObject);

    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

  private
    FRealPgCount, FCurPage: Integer;
    FActivating:   Boolean;
    FMadeSettings: Boolean;
    FIconID:       Integer;

    procedure SetMode (Value: Boolean);
    procedure UpdatePageInfo;
    procedure UpdateRecordInfo;
    procedure ActivateMoving;
    procedure ShowProcessing;

    procedure WMGetMinMaxInfo (var Message: TWMGetMinMaxInfo);
      message WM_GETMINMAXINFO;
    procedure WMSysCommand(var Message: TWMSysCommand); message WM_SYSCOMMAND;
    procedure UMActivateIcon (var Message: TMessage); message UM_ACTIVATEICON;

  public
    Crystal1:      TCrystal;
    constructor Create(AOwner: TComponent; Crst: TCrystal);
    destructor Destroy; override;

    function Made: Boolean;
  end;

var
  frmCrystal: TfrmCrystal;

implementation

const
  Min_Width  = 319;
  Min_Height = 123;

{$R *.DFM}

{*************
* Name.......: Create
*}
constructor TfrmCrystal.Create(AOwner: TComponent; Crst: TCrystal);
begin
  FMadeSettings := False;
  inherited Create(AOwner);

  Crystal1 := TCrystal.Create(Self);
  Crystal1.Assign(Crst);

  // Form Initialization
  with Crystal1 do
  begin
    Self.Caption      := WindowTitle;
    Self.WindowState  := Crystal1.WindowState;
    Self.BorderStyle  := WindowBorderStyle;

    Self.BorderIcons  := [];
    if WindowControlBox then Self.BorderIcons := Self.BorderIcons + [biSystemMenu];
    if WindowMaxButton  then Self.BorderIcons := Self.BorderIcons + [biMaximize];
    if WindowMinButton  then Self.BorderIcons := Self.BorderIcons + [biMinimize];

    if ( Crystal1.WindowState <> wsMaximized ) then
      begin
        Self.ClientWidth  := WindowWidth;
        Self.ClientHeight := WindowHeight + pnlControls.Height;
      end
    else
      begin
        WindowWidth       := Self.ClientWidth;
        WindowHeight      := Self.ClientHeight - pnlControls.Height;
      end;

    WindowLeft        :=  0;
    WindowTop         :=  pnlControls.Top + pnlControls.Height;
  end;

  // Default parameters setting
  FRealPgCount := 1;
  FCurPage     := 1;
  FActivating  := False;
  FMadeSettings := True;
  FormResize(Self);
  SetMode(False);
  // Begin Reading information
  //Timer1.OnTimer  := TimeReading;
end;


destructor TfrmCrystal.Destroy;
begin
  Timer1.OnTimer := nil;
  Crystal1.Free;
  //MyTaskBarDeleteIcon(Handle, FIconID);
  //MyTaskBarDeleteButton(Handle);
  inherited Destroy;
end;


function TfrmCrystal.Made: Boolean;
begin
  if not Visible then
    Result := True
  else
    Result := (not Assigned(Timer1.OnTimer)) and (not lblProc.Visible) and (btnGo.Enabled);
end;


//-------------  Buttons Events  -----------//

{*************
* Name.......: btnPrint (on Click Event)
* Description: Print with selection interval (if need)
*}
procedure TfrmCrystal.btnPrintClick(Sender: TObject);
begin
  with Crystal1 do
  begin
    // Show printer setup dialog
    if UserPrinterSetup then
      PrinterSetupDialog1.Execute;

    // Show Pages interval selection dialog
    if UserPrinterOptions then
      begin
        if ( PrinterCopies > 0 ) then
          PrintDialog1.Copies := PrinterCopies
        else
          PrintDialog1.Copies := 1;

        PrintDialog1.Collate  := PrinterCollation;

        if ( (PrinterStartPage > 1) and (PrinterStartPage <= FRealPgCount) ) then
          PrintDialog1.FromPage := PrinterStartPage
        else
          PrintDialog1.FromPage := 1;

        if ( (PrinterStopPage < PrintDialog1.FromPage) or (PrinterStopPage > FRealPgCount) ) then
          PrintDialog1.ToPage := FRealPgCount
        else
          PrintDialog1.ToPage := PrinterStopPage;

        PrintDialog1.MinPage  := 1;
        PrintDialog1.MaxPage  := FRealPgCount;

        if PrintDialog1.Execute then
        begin
          // Reset options on User selection
          PrinterCopies    := PrintDialog1.Copies;
          PrinterCollation := PrintDialog1.Collate;
          PrinterStartPage := PrintDialog1.FromPage;
          PrinterStopPage  := PrintDialog1.ToPage;

          Destination      := toPrinter;
          IsPreview        := False;

          SetMode(False);
          Timer1.OnTimer := TimePrinting;
          PrintFromWindow;

          Destination      := toWindow;
          IsPreview        := True;
        end;
      end
    else
      begin
        Destination      := toPrinter;
        IsPreview        := False;

        SetMode(False);
        Timer1.OnTimer := TimePrinting;
        PrintFromWindow;

        Destination      := toWindow;
        IsPreview        := True;
      end;
  end;
end;

{*************
* Name.......: btnExport (on Click event)
* Description: To future
*}
procedure TfrmCrystal.btnExportClick(Sender: TObject);
begin
  //
end;

{*************
* Name.......: btnMagnification (on Click event)
* Description: Resize window
*}
procedure TfrmCrystal.btnMagnClick(Sender: TObject);
begin
  Crystal1.NextMagnification;
end;


{*************
* Name.......: btnFirst (on Click event)
* Description: Show first page
*}
procedure TfrmCrystal.btnFirstClick(Sender: TObject);
begin
  SetMode(False);
  btnSpBreak.Visible := True;
  btnBreak.Visible   := False;

  Timer1.OnTimer   := TimeMoving;
  Crystal1.ShowFirst;
end;


{*************
* Name.......: btnPrior (on Click event)
* Description: Show Prior page
*}
procedure TfrmCrystal.btnPriorClick(Sender: TObject);
begin
  SetMode(False);
  btnSpBreak.Visible := True;
  btnBreak.Visible   := False;
  Timer1.OnTimer   := TimeMoving;
  Crystal1.ShowPrior;
end;


{*************
* Name.......: btnNext (on Click event)
* Description: Show Next page
*}
procedure TfrmCrystal.btnNextClick(Sender: TObject);
begin
  SetMode(False);
  btnSpBreak.Visible := True;
  btnBreak.Visible   := False;
  Timer1.OnTimer   := TimeMoving;
  Crystal1.ShowNext;
end;


{*************
* Name.......: btnSpecialLast (on Click event)
* Description: Show Last page by especial method
*}
procedure TfrmCrystal.btnSpLastClick(Sender: TObject);
begin
  SetMode(False);
  btnSpBreak.Visible := True;
  btnBreak.Visible   := False;
  Timer1.OnTimer   := TimeMoving;
  Crystal1.ShowPage(FRealPgCount);
end;

{*************
* Name.......: btnSetPage (on Click event)
* Description: Move to selected page
*}
procedure TfrmCrystal.btnSetPageClick(Sender: TObject);
begin
  SetMode(False);
  btnSpBreak.Visible := True;
  btnBreak.Visible   := False;
  Timer1.OnTimer     := TimeMoving;
  Crystal1.ShowPage(seNum.Value);
end;


{*************
* Name.......: btnBreak (on Click event)
* Description: break current operation
*}
procedure TfrmCrystal.btnBreakClick(Sender: TObject);
begin
  btnBreak.Enabled := False;
  btnBreak.Visible := True;
  btnSpBreak.Visible := False;
  Timer1.OnTimer   := TimeBreaking;
  Crystal1.BreakOperation;// Break Reading records of generating page
end;

//-------------  On Timer Methods ----------------//

{*************
* Name.......: TimeReading
* Description: Updating form in reading mode
*}
procedure TfrmCrystal.TimeReading(Sender: TObject);
begin
  ShowProcessing;
  if Crystal1.IsCompleted then
    begin  // End of Reading mode
      UpdateRecordInfo;
      Timer1.OnTimer := nil;
      ActivateMoving;
    end
  else     // Update controls
    begin
      // if ( Self.ClientWidth  <> Crystal1.WindowWidth ) then FormResize(Self);
      UpdateRecordInfo;
    end;
end;


{*************
* Name.......: TimeMoving
* Description: Updating form in moving mode
*}
procedure TfrmCrystal.TimeMoving(Sender: TObject);
begin
  Timer1.Enabled  := False;
  Application.ProcessMessages;
  lblProc.Caption := 'Going to page...';
  ShowProcessing; // update controls
  if Crystal1.IsCompleted then
  begin           // End of moving mode
    Timer1.OnTimer := nil;
    UpdatePageInfo;
    SetMode(True);
  end;
  Timer1.Enabled := True;
end;


{*************
* Name.......: TimeBreaking
* Description: Updating form in breaking mode
*}
procedure TfrmCrystal.TimeBreaking(Sender: TObject);
begin
  ShowProcessing; // update controls
  lblProc.Caption := 'Breaking...';
  if Crystal1.IsCompleted then
  begin           // end of breaking mode
    btnBreak.Enabled   := False;
    btnBreak.Visible   := True;
    btnSpBreak.Visible := False;
    Timer1.OnTimer     := TimeMoving;
    Crystal1.ShowPage(FCurPage);
  end;
end;


{*************
* Name.......: TimeActivating
* Description: Especial method for correction bug in DLL (in calculating pages count)
*}
procedure TfrmCrystal.TimeActivating(Sender: TObject);
begin
  lblProc.Caption := 'Generating Last Page...';
  if FActivating then
    begin        // Reselect flag of calls count
      FActivating     := False;
      lblCurRec.Hint  := lblInfo.Caption;
      Crystal1.BreakOperation;
      ShowProcessing;
    end
  else if Crystal1.IsCompleted then // waiting breaking operation
    begin                           // Return on 1st page
      ShowProcessing;
      btnBreak.Enabled   := False;
      btnBreak.Visible   := True;
      btnSpBreak.Visible := False;
      lblCurRec.Hint     := '';
      Timer1.OnTimer     := TimeMoving;
      Crystal1.ShowFirst;
    end
  else
    ShowProcessing;                 // update form controls
end;


{*************
* Name.......: TimePrinting
* Description: Updating form in printing mode
*}
procedure TfrmCrystal.TimePrinting(Sender: TObject);
begin
  lblProc.Caption := 'Printing ...';
  ShowProcessing;
  with Crystal1.ReadStatus do
  if ( PrintEnded or (Abs(NumRecordsPrinted - NumRecordsSelected) < 2) ) then
  begin   // End of printing mode - !! very bad condition - also library bug
    Timer1.OnTimer   := nil;
    UpdatePageInfo;
    SetMode(True);
  end
end;


{*************
* Name.......: ShowProcessing
* Description: Update process labels
*}
procedure TfrmCrystal.ShowProcessing;
begin
  // Show message
  lblProc.Visible := True;
  // Flash message
  if ( lblProc.Font.Color = clBlue ) then
    lblProc.Font.Color := clBtnFace
  else
    lblProc.Font.Color := clBlue;
end;

//--------------   Private methods   --------------//

{*************
* Name.......: SetMode
* Description: Enabling/Disabling controls
*}
procedure TfrmCrystal.SetMode (Value: Boolean);
begin
  btnBreak.Enabled   := False;
  btnBreak.Visible   := True;
  btnSpBreak.Visible := False;
  btnFirst.Enabled   := Value and (FCurPage > 1);
  btnPrior.Enabled   := Value and (FCurPage > 1);
  btnNext.Enabled    := Value and (FCurPage < FRealPgCount);
  btnSpLast.Enabled  := Value and (FCurPage < FRealPgCount);
  btnPrint.Enabled   := Value ;
  btnExport.Enabled  := False;//Value ; To Future
  btnMagn.Enabled    := Value ;
  seNum.Enabled      := Value ;
  btnGo.Enabled      := Value ;
  lblPages.Visible   := Value;
  lblPage.Visible    := Value;

  pnlControls.Refresh;
end;

{*************
* Name.......: UpdatePageInfo
*}
procedure TfrmCrystal.UpdatePageInfo;
begin
  lblProc.Visible     := False;
  btnBreak.Enabled    := False;
  btnBreak.Visible    := True;
  btnSpBreak.Visible  := False;

  with Crystal1.ReadStatus do
  begin
    FCurPage     := DisplayPageN;
    FRealPgCount := LatestPageN;
  end;

  lblPages.Caption    := IntToStr(FCurPage) + ' of '
    + IntToStr(FRealPgCount);
  seNum.Value := FCurPage;

  lblPages.Visible := True;
  lblPage.Visible  := True;
end;


{*************
* Name.......: UpdateRecordInfo
*}
procedure TfrmCrystal.UpdateRecordInfo;
begin
  with Crystal1.ReadStatus do
    lblCurRec.Caption   := IntToStr(NumRecordsRead);
end;


{*************
* Name.......: ActivateMoving
* Description: Make particular correction library bug which
*              forbide normal moving
*}
procedure TfrmCrystal.ActivateMoving;
var
  NeedActivate: Boolean;
  FPageCount: Integer;

begin
  // Warning !! Not change order of the operators!
  // Need in begining calculate the LibraryPageCount
  // and only after that counting real value!!
  FPageCount     := Crystal1.CalcPagesCount;
  FRealPgCount   := Crystal1.ReadStatus.LatestPageN;
  //------------------------------------------------//

  NeedActivate   := not (FPageCount = FRealPgCount);
  seNum.MaxValue := FRealPgCount;

  if NeedActivate and (FRealPgCount > 1) then
    begin
      FActivating        := True;
      btnBreak.Enabled   := False;
      btnBreak.Visible   := True;
      btnSpBreak.Visible := False;
      Timer1.OnTimer     := TimeActivating;
      Crystal1.ShowNext;
    end
  else
    begin // not need using the method
      UpdatePageInfo;
      SetMode(True);
    end;
end;

//-------------------- Form methods ------------------//

procedure TfrmCrystal.FormResize(Sender: TObject);
var
  TheHandle: HWnd;

begin
  if ( Width > Min_Width + 235 ) then
    begin
      pnlControls.Height := 34;
      lblPage.Left := Min_Width -10;
      lblPage.Top  := 9;
      lblPages.Left := Min_Width + 35;
      lblPages.Top  := 9;
      lblProc.Left := Min_Width - 10;
      lblProc.Top  := 9;
      lblTotal.Left := Min_Width + 157;
      lblTotal.Top  := 9;
      lblCurRec.Left := Min_Width + 205;
      lblCurRec.Top  := 9;
    end
  else
    begin
      pnlControls.Height := 53;
      lblPage.Left   := 6;
      lblPage.Top    := 33;
      lblPages.Left  := 51;
      lblPages.Top   := 33;
      lblProc.Left   := 6;
      lblProc.Top    := 33;
      lblTotal.Left  := 175;
      lblTotal.Top   := 33;
      lblCurRec.Left := 223;
      lblCurRec.Top  := 33;
    end;

  if not FMadeSettings then Exit;
  TheHandle := Crystal1.GetWindowHandle;

  if ( TheHandle <> 0 ) then
    MoveWindow (
      TheHandle,	// handle of window
      0,	// horizontal position
      pnlControls.Top + pnlControls.Height,	// vertical position
      Self.ClientWidth,	// width
      Self.ClientHeight - pnlControls.Height,	// height
      True 	// repaint flag
     );
end;


{*************
* Name.......: WMGetMinMaxInfo
*}
procedure  TfrmCrystal.WMGetMinMaxInfo (var Message: TWMGetMinMaxInfo);
var
  MinMax: PMinMaxInfo;

begin
  inherited;
  MinMax := Message.MinMaxInfo;
  MinMax^.ptMinTrackSize.X := 319;
  MinMax^.ptMinTrackSize.Y := 123;
end;


procedure TfrmCrystal.WMSysCommand(var Message: TWMSysCommand);
begin
  if ( (
         ((Message.CmdType and $FFF0) <> SC_MINIMIZE) and
         ((Message.CmdType and $FFF0) <> SC_RESTORE)
       ) or (biMinimize in BorderIcons)
     ) then
    inherited;
end;


procedure TfrmCrystal.FormShow(Sender: TObject);
begin
  Crystal1.ExecuteOnly;
  Timer1.OnTimer  := TimeReading;
  FIconID := StrToInt(FormatDateTime('hhnnss', Now));
  //MyTaskBarAddIcon(Handle, FIconID, Application.Icon.Handle, LPSTR('Crystal Report'));
  //MyTaskBarAddButton(Handle);
end;

procedure TfrmCrystal.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ( Key = vk_ESCAPE ) then Close;
end;

procedure TfrmCrystal.UMActivateIcon (var Message: TMessage);
begin
  // SetFocus;
end;

procedure TfrmCrystal.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
